import {
    createSlice,
    createAsyncThunk,
    PayloadAction,
    isRejectedWithValue,
} from "@reduxjs/toolkit"
import axios from "axios"
import { backendHttp, apiHttp } from "../../lib/https"
import { toast } from "react-toastify"

// Define types for state
interface User {
    map(
        arg0: (
            admin: { name: any; role: any },
            index: number
        ) => { srno: number; name: any; role: any }
    ): unknown
    id: string
    email: string
}

interface Admin {
    id: string
    name: string
    role: string
}
interface AuthState {
    user: User | null
    admins: Admin[]
    isAuthenticated: boolean
    isLoading: boolean
    error: string | null
}

// Async thunk for login
export const loginUser = createAsyncThunk<
    User,
    { email: string; password: string },
    { rejectValue: string }
>("auth/login", async ({ email, password }, { rejectWithValue }) => {
    try {
        const response = await backendHttp.post("admin/login", {
            email,
            password,
        })
        localStorage.setItem("authToken", response.data?.data?.token) // Save token
        toast.success(response.data?.message)
        return response.data
    } catch (error: any) {
        return rejectWithValue(
            error.response?.data?.message || "An error occurred"
        )
    }
})

// Async thunk for register
export const registerUser = createAsyncThunk<
    User,
    { email: string; password: string },
    { rejectValue: string }
>("auth/register", async (data, { rejectWithValue }) => {
    try {
        const response = await axios.post(
            "https://health-digi.dmlabs.in/auth/register",
            data
        )
        return response.data
    } catch (error: any) {
        return rejectWithValue(
            error.response?.data?.message || "An error occurred"
        )
    }
})

export const adminList = createAsyncThunk<
    Admin[],
    void,
    { rejectValue: string }
>("/auth", async (_, { rejectWithValue }) => {
    try {
        const response = await apiHttp.get("/auth")
        console.log(response?.data?.data)
        return response?.data?.data?.map(
            (admin: { id: string; name: string; role: string }) => ({
                id: admin.id,
                name: admin?.name,
                role: admin?.role || "N/A",
            })
        )
        console.log(response.data.data)
    } catch (error: any) {
        return rejectWithValue(
            error.response?.data?.message || "An error occurred"
        )
    }
})

export const deleteAdmin = createAsyncThunk<
    string,
    string,
    { rejectValue: string }
>("deleteAdmin", async (id, { rejectWithValue }) => {
    try {
        const response = await apiHttp.delete(`/auth/${id}`)
        // console.log(response, "response of deletion")
        return id // Returning the deleted admin's ID
    } catch (error: any) {
        return rejectWithValue(
            error.response?.data?.message || "An error occurred"
        )
    }
})

// export const putAdmin = createAsyncThunk<
//     Admin, // Return type (updated admin object)
//     { id: string; data: Partial<Admin> }, // Argument type
//     { rejectValue: string } // Rejection type
// >(
//     "updateAdmin",
//     async ({ id, data }, { rejectWithValue }) => {
//         try {
//             const response = await apiHttp.put(`/auth/${id}`, data)
//             return response.data // Ensure response contains the updated admin object
//         } catch (error: any) {
//             return rejectWithValue(
//                 error.response?.data?.message || "An error occurred"
//             )
//         }
//     }
// )

const initialState: AuthState = {
    user: null,
    admins: [],
    isAuthenticated: false,
    isLoading: false,
    error: null,
}

const authSlice = createSlice({
    name: "auth",
    initialState,
    reducers: {
        logout: (state) => {
            state.user = null
            state.isAuthenticated = false
        },
    },
    extraReducers: (builder) => {
        builder
            // Login
            .addCase(loginUser.pending, (state) => {
                state.isLoading = true
                state.error = null
            })
            .addCase(
                loginUser.fulfilled,
                (state, action: PayloadAction<User>) => {
                    state.isLoading = false
                    state.isAuthenticated = true
                    state.user = action.payload
                }
            )
            .addCase(
                loginUser.rejected,
                (state, action: PayloadAction<string | undefined>) => {
                    state.isLoading = false
                    state.error = action.payload || "An error occurred"
                }
            )
            // Register
            .addCase(registerUser.pending, (state) => {
                state.isLoading = true
                state.error = null
            })
            .addCase(
                registerUser.fulfilled,
                (state, action: PayloadAction<User>) => {
                    state.isLoading = false
                    state.isAuthenticated = true
                    state.user = action.payload
                }
            )
            .addCase(
                registerUser.rejected,
                (state, action: PayloadAction<string | undefined>) => {
                    state.isLoading = false
                    state.error = action.payload || "An error occurred"
                }
            )

            // created by Jaanvi and Eknoor
            //AdminList
            .addCase(adminList.pending, (state) => {
                state.isLoading = true
                state.error = null
            })
            .addCase(
                adminList.fulfilled,
                (state, action: PayloadAction<Admin[]>) => {
                    state.isLoading = false
                    state.admins = action.payload
                }
            )

            .addCase(
                adminList.rejected,
                (state, action: PayloadAction<string | undefined>) => {
                    state.isLoading = false
                    state.error = action.payload || "An error occurred"
                }
            )

            .addCase(deleteAdmin.pending, (state) => {
                state.isLoading = true
            })
            .addCase(deleteAdmin.fulfilled, (state, action) => {
                state.isLoading = false
                state.admins = state.admins.filter(
                    (admin) => String(admin.id) !== String(action.payload)
                )
            })
            .addCase(deleteAdmin.rejected, (state, action) => {
                state.isLoading = false
                state.error = action.payload || "Failed to delete admin"
            })
            //update Admin

            // .addCase(putAdmin.pending, (state) => {
            //     state.isLoading = true
            //     state.error = null
            // })
            // .addCase(putAdmin.fulfilled, (state, action: PayloadAction<Admin>) => {
            //     state.isLoading = false
            //     state.admins = state.admins.map((admin) =>
            //         admin.id === action.payload.id ? action.payload : admin
            //     )
            // })
            // .addCase(putAdmin.rejected, (state, action: PayloadAction<string | undefined>) => {
            //     state.isLoading = false
            //     state.error = action.payload || "Failed to update admin"
            // })
            
    },
})

export const { logout } = authSlice.actions
export default authSlice.reducer
